
DEFAULT_RATE = 500  -- Times per second
SLOW_RATE = 80
SLOWER_RATE = 20
MAX_LIGHTS = 20

function init()
	checkTime = 0
	isTriggered = false    -- If false then the script won't do anything, to improve overall performance
	isBroken = false
	updateRate = DEFAULT_RATE -- Number of times per second to change light positions
	rateChangeTime = 0
	runTrigger = FindTrigger("run-computer-room", true)
	panels = {}
	local panelShapes = FindShapes("panel")
	for i=1, #panelShapes do
		local min, max = GetShapeBounds(panelShapes[i])
		local transform = GetShapeWorldTransform(panelShapes[i])
		-- Move points away from shape a bit so the light can shine on the surface rather than
		-- appearing partially inside
		min = VecAdd(TransformToLocalPoint(transform, min), Vec(0, 0, -0.1))
		max = VecAdd(TransformToLocalPoint(transform, max), Vec(0, 0, -0.1))
		min = TransformToParentPoint(transform, min)
		max = TransformToParentPoint(transform, max)
		panels[i] = {
			shape = panelShapes[i],
			min = min,
			max = max,
			currentIndex = 1,
			accumulatedTime = 0,
			lightPositions = {}
		}
		for j=1, MAX_LIGHTS do
			panels[i].lightPositions[j] = Vec(0, 0, 0)
		end
	end
	computerSound = LoadLoop("MOD/sounds/alien-mainframe.ogg")
	mainShape = FindShape("computer")
	local min, max = GetShapeBounds(mainShape)
	local boundsSize = VecSub(max, min)
	bodyCenter = VecLerp(min, max, 0.5)
end

function tick(deltaTime)
	checkTime = checkTime + deltaTime
	if checkTime > 2 then
		isTriggered = checkTrigger()
		checkTime = 0
	end
	if isTriggered and not isBroken then
		-- TODO: Periodically reduce update rate
		--rateChangeTime = rateChangeTime + deltaTime
		--if rateChangeTime % 20 < 4 then
		--	if rateChangeTime % 1 < 0.2 then
		--		updateRate = 20
		--	else
		--		updateRate = 50
		--	end
		--else
		--	updateRate = DEFAULT_RATE
		--end
		
		-- Blinkenlights
		for i=1, #panels do
			if IsShapeBroken(panels[i].shape) then
				isBroken = true
			end
			updateLightPositions(panels[i], deltaTime * updateRate)
			showLights(panels[i].lightPositions)
		end
		
		PlayLoop(computerSound, bodyCenter, 0.1)
	end
end

function checkTrigger()
	return IsPointInTrigger(runTrigger, GetPlayerTransform().pos)
end

function showLights(points)
	for i=1, #points do
		PointLight(points[i], 0, 1, 0.6, 1.0)
	end
end

function updateLightPositions(panel, updateAmount)
	panel.accumulatedTime = panel.accumulatedTime + updateAmount
	while panel.accumulatedTime >= 1 do
		local pos = randomVec(panel.min, panel.max)
		panel.lightPositions[panel.currentIndex] = pos
		panel.currentIndex = (panel.currentIndex % #panel.lightPositions) + 1
		panel.accumulatedTime = panel.accumulatedTime - 1
	end
end


function randomFloat(lower, upper)
	return lower + math.random() * (upper - lower);
end

function randomVec(min, max)
	return Vec(randomFloat(min[1], max[1]), randomFloat(min[2], max[2]), randomFloat(min[3], max[3]))
end