
function init()
	checkTime = 0
	isTriggered = false    -- If false then the script won't do anything, to improve overall performance
	isBroken = false       -- True if the object has broken
	
	runTriggers = FindTriggers("run")
	startPos = GetLocationTransform(FindLocation("start")).pos
	endPos = GetLocationTransform(FindLocation("end")).pos
	center = VecLerp(startPos, endPos, 0.5)
	mainShape = FindShape("main-body")
	buzzSound = LoadLoop("light/buzz1.ogg")
	sparkSound = LoadSound("light/spark0.ogg")
	
	points = {}
	maxIterations = 4
	updateTime = 0;
end

function tick(deltaTime)
	checkTime = checkTime + deltaTime
	updateTime = updateTime + deltaTime
	if checkTime > 1 then
		isTriggered = checkTrigger()
		checkTime = 0
	end
	if isTriggered and not isBroken then
		if IsShapeBroken(mainShape) then
			blowUpEmitters()
		end

		if updateTime > 0.05 then
			updateTime = 0
			points = {}
			updatePoints(startPos, endPos, 1)
			table.insert(points, endPos)
		end
		drawLines()
		PlayLoop(buzzSound, center, 0.8)
	end
end

function updatePoints(p1, p2, iteration)
	if iteration > maxIterations then
		table.insert(points, p1)
		return
	end
	local length = VecLength(VecSub(p2, p1))
	local mid = VecLerp(p1, p2, 0.5)
	local offset = VecScale(Vec(randomFloat(-1, 1), 0, randomFloat(-1, 1)), length / 8)
	mid = VecAdd(mid, offset)
	updatePoints(p1, mid, iteration + 1)
	updatePoints(mid, p2, iteration + 1)
end

function drawLines()
	local p1 = points[1]
	PointLight(p1, 0, 1, 0.2, 0.2)
	for i=2, #points do
		local p2 = points[i]
		DrawLine(p1, p2, 0, 1, 0.2, 1.0)
		PointLight(p2, 0, 1, 0.2, 0.2)
		p1 = p2
	end
end

function blowUpEmitters()
	isBroken = true

	ParticleReset()
	ParticleType("plain")
	ParticleTile(4)   -- Plain circle texture
	ParticleColor(1.0, 1.0, 1.0, 0.8, 0.5, 0.0)
	ParticleEmissive(5, 0)
	ParticleRadius(0.01)
	ParticleGravity(-0.5)
	ParticleCollide(1, 1, "constant", 0.2)
	
	PlaySound(sparkSound, startPos, 1.0)
	PlaySound(sparkSound, endPos, 1.0)
	for i=1, 100 do
		SpawnParticle(startPos, randomVec(Vec(-2, -2, -2), Vec(2, 2, 2)), randomFloat(0.2, 1.0))
		SpawnParticle(endPos, randomVec(Vec(-2, -2, -2), Vec(2, 2, 2)), randomFloat(0.2, 1.0))
	end
end

function checkTrigger()
	local playerPos = GetPlayerTransform().pos
	return IsPointInTrigger(runTriggers[1], playerPos) or
	       IsPointInTrigger(runTriggers[2], playerPos)
end


function randomFloat(lower, upper)
	return lower + math.random() * (upper - lower);
end

function randomVec(min, max)
	return Vec(randomFloat(min[1], max[1]), randomFloat(min[2], max[2]), randomFloat(min[3], max[3]))
end