
function init()
	checkTime = 0
	isTriggered = false    -- If false then the script won't do anything, to improve overall performance
	isBroken = false       -- True if an emitter has broken
	runTrigger = FindTrigger("run")
	startPos = GetLocationTransform(FindLocation("start")).pos
	endPos = GetLocationTransform(FindLocation("end")).pos
	emitters = FindShapes("emitter")
	light = FindLight()
	beamLength = VecLength(VecSub(startPos, endPos)) - 0.2  -- For raycast
	beamDirection = VecNormalize(VecSub(endPos, startPos))  -- Unit vector representing direction
	hummSound = LoadLoop("light/buzz3.ogg")
	sparkSound = LoadSound("light/spark0.ogg")
end

function tick(deltaTime)
	checkTime = checkTime + deltaTime
	if checkTime > 1 then
		isTriggered = checkTrigger()
		checkTime = 0
	end
	if isTriggered and not isBroken then
		if IsShapeBroken(emitters[1]) then
			blowUpEmitter(emitters[1])
		elseif IsShapeBroken(emitters[2]) then
			blowUpEmitter(emitters[2])
		end

		DrawLine(startPos, endPos, 0, 1, 0.6, 1.0)
		
		PlayLoop(hummSound, startPos, 0.8)
		PlayLoop(hummSound, endPos, 0.8)

		-- Destroy anything in it's path
		local hit, dist, normal, shape = QueryRaycast(startPos, beamDirection, beamLength, 0.1)
		if hit then
			local hitPoint = VecAdd(startPos, VecScale(beamDirection, dist))
			MakeHole(hitPoint, 0.2, 0.2, 0.2, true)
		end
	end
end

function blowUpEmitter(emitter)
	isBroken = true
	SetLightEnabled(light, false)

	local min, max = GetShapeBounds(emitter)
	local boundsSize = VecSub(max, min)
	local center = VecLerp(min, max, 0.5)
	PlaySound(sparkSound, center, 1.0)
	ParticleReset()
	ParticleType("plain")
	ParticleTile(4)   -- Plain circle texture
	ParticleColor(1.0, 1.0, 1.0, 0.8, 0.5, 0.0)
	ParticleEmissive(5, 0)
	ParticleRadius(0.01)
	ParticleGravity(-0.5)
	ParticleCollide(1, 1, "constant", 0.2)
	for i=1, 100 do
		SpawnParticle(center, randomVec(Vec(-2, -2, -2), Vec(2, 2, 2)), randomFloat(0.2, 1.0))
	end
end

function checkTrigger()
	return IsPointInTrigger(runTrigger, GetPlayerTransform().pos)
end


function randomFloat(lower, upper)
	return lower + math.random() * (upper - lower);
end

function randomVec(min, max)
	return Vec(randomFloat(min[1], max[1]), randomFloat(min[2], max[2]), randomFloat(min[3], max[3]))
end